///////////////////
// Control.java //
/////////////////

//java
import java.util.Timer;
import java.util.TimerTask;
import java.io.IOException;
import java.text.DecimalFormat;


import net.tinyos.message.Message;
import net.tinyos.message.MessageListener;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.PrintStreamMessenger;
//
//imports para el xml (uso librería JDOM de jdom.org
//
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

//propias generadas por MIG
//import tfc.*;



public class ControlMotas extends TimerTask implements MessageListener{

	//variables
	public static ControlMotas instance = null;
	static GUI gui;
	private static Timer timer= new Timer();

	MoteIF mifR; //recibir mensajes
	static MoteIF mifE; //enviar mensajes
	
	
	//
	//Se inicia todo
	//
	public static void main (String args[]){
	gui=new GUI();
	println ("Control de Motas, iniciando interface...");
	gui.lanzarFrame();
	
	instance = new ControlMotas();
	
	PhoenixSource phoenix;
	phoenix=BuildSource.makePhoenix(PrintStreamMessenger.err);
	
	String source="sf@localhost:9002";
	phoenix=BuildSource.makePhoenix(source, PrintStreamMessenger.err);
	
	instance.mifR=new MoteIF(phoenix);
	ControlMotas.mifE=new MoteIF(phoenix);
	
	instance.mifR.registerListener(new SensorMsg(), instance);

	timer.scheduleAtFixedRate(instance, 0, Long.parseLong(gui.devuelveTimerSend())*10);
	}
	
	//modificamos el println para que lo haga en nuestro entorno gráfico 
	private static void println (String cadena){
		gui.Imprimir(cadena);
	}
	
	//
	//Control mensajes recibidos de mota
	//
	public void messageReceived (int para, Message mensaxe){
		
		if (mensaxe instanceof SensorMsg){
			SensorMsg sm = (SensorMsg) mensaxe;	// mensaje de la mota
			double voltaje = ((double) sm.get_cBat()*2)/1024 *((double)sm.get_vRef()/1000); //convertimos a voltios
			double temperatura = (double) sm.get_cTemp()/1024 *((double)sm.get_vRef()/1000); //convertimos inicial temperatura
			temperatura = (temperatura-0.5)/0.01; //conversión a celsius.
			gestionAlarma(temperatura);//gestiona si hay alarma de temperatura.
			gui.MostrarDatos(formatea(voltaje), formatea(temperatura));  //actualizar datos en GUI.
			if (gui.activoXML()==true) {
				actualizarXML(formatea(voltaje), formatea(temperatura));//está activado el checkbox del GUI así que actualizo el Xml
			}
		}
	}
	
	//
	//Formateamos los datos a string desde double y con aspecto bonito
	//
	private String formatea(double dato){
		DecimalFormat datoFormateado= new DecimalFormat("#0.00");
		return datoFormateado.format(dato);
	}
	
	//
	//Actualizamos los tiempos de lectura y envío de la mota
	//
	
	public static void actualizarTR (String tr){		//el tiempo de lectura tr
		TimerReadMsg msx= new TimerReadMsg();
		msx.set_timerVal(Long.parseLong(tr));
		try {
			mifE.send(1,msx);
			println("actualizando el tiempo de lectura a "+ tr + "milisegundos");
		} catch (IOException e) {
			println("Hubo un error, inténtalo más tarde, no se pudo enviar el mensaje");
		}
	}
	
	public static void actualizarTS (String ts){		//el tiempo de envío ts
		TimerSendMsg msx= new TimerSendMsg();
		msx.set_timerVal(Long.parseLong(ts));
		try {
			mifE.send(1,msx);
			println("actualizando el tiempo de envío a "+ ts + "milisegundos");
		} catch (IOException e) {
			println("Hubo un error, inténtalo más tarde, no se pudo enviar el mensaje");
		}
	}	
	
	//
	//Gestión de alarma de temperatura
	//
	public void gestionAlarma (double tempActual){
		double tempAlarma = gui.devuelveTempAlarm();
		
		
		if (tempActual> tempAlarma) {   // T mota es mayor que la de alarma.
			gui.alarmaTemperatura(true);
			println("Alarma de TEMPERATURA, superamos los "+tempAlarma+"ºC");
		}
		else {
			gui.alarmaTemperatura(false);	//no hay alarma
			println("No hay alertas");
		}
		
	}

	//
	//Gestión del fichero XML
	//queda construido así para facilitar en el futuro actualizaciones y adiciones al fichero de salida
	//podría simplificarse
	//ahora ponemos el número de mota fijo a 1, ya que solo tengo 1 para las pruebas
	//
	public void actualizarXML (String volt, String temp){
		try {
			 
			SAXBuilder builder = new SAXBuilder();
			//obtengo trayectoria de mi $HOME
			String ruta=System.getenv("HOME");
			ruta=ruta+"/workspace/web/motas.xml";
			
			File xmlFile = new File(ruta);
	 
			Document doc = (Document) builder.build(xmlFile); // obtengo documento
			Element rootNode = doc.getRootElement();	// obtengo nodo raíz que es "<xml>"
			//obtengo nodo hijo que es la mota
			Element mota = rootNode.getChild("mota");
			//actualizo los valores
			mota.getChild("numero").setText("1");//numero
			mota.getChild("temperatura").setText(temp);//temperatura
			mota.getChild("voltaje").setText(volt);//voltaje
			//le doy salida
			XMLOutputter xmlOutput = new XMLOutputter();	 
			// le da formato indentado
			xmlOutput.setFormat(Format.getPrettyFormat());
			xmlOutput.output(doc, new FileWriter(ruta));
	 
			// xmlOutput.output(doc, System.out);

		  } catch (IOException io) {
			io.printStackTrace();
		  } catch (JDOMException e) {
			e.printStackTrace();
		  }
		
	}	
	
	
	
	//
	//COMENZAMOS EL THREAD
	//
	
	public void run(){
		}
	}
